
#include <stdio.h>
#include <windows.h>  /* only needed for MessageBox() which outputs errors */
#include "globals.h"  /* only needed for Save_Cor_aIBI which is flag for writing to file */

#define BUFFER_SIZE			64		// should always be sufficient to hold 6-10 seconds of data (current buffer time)
#define DEFAULT_THRESHOLD	100		// default (non-statistical) theshold for error detection
#define MIN_TRANSMIT_IBI	1		// smallest value that transmitter would ever send
#define MAX_TRANSMIT_IBI	1500	// largest value that transmitter would ever send
#define MIN_ACTUAL_IBI		200		// smallest value an IBI could ever really have
#define MAX_ACTUAL_IBI		1450	// largest value an IBI could ever really have

			/* this is based on Jeromie's work - see his thesis for details */
#define HISTORY_SIZE		5		// count of most recently corrected aIBI's used to calculate statistics
#define MULTIPLIER_1		10		// for determining threshold for detecting error (based on training data)
#define MULTIPLIER_2		15		// for determining threshold for accepting correction (based on training data)
#define THRESH_MIN_DETECT	10		// for determining threshold for detecting error (based on training data)
#define THRESH_MAX_DETECT	300		// for determining threshold for detecting error (based on training data)
#define THRESH_MIN_ACCEPT	100		// for determining threshold for accepting correction (based on training data)
#define THRESH_MAX_ACCEPT	100		// for determining threshold for accepting correction (based on training data)

int					  ibi_buffer[BUFFER_SIZE];	/* local aIBI buffer */
int					  buffer_size;				/* count of aIBI's in buffer */
int					  buffer_total_time;		/* total time of aIBI's in buffer */
int					  check_index;				/* current index of buffer to be checked */
													/* everything under this is assumed ok */
int					  eat_index;				/* current index of consumer of the buffer */
int					  eat_delta;				/* portion of aIBI time already consumed at eat_index */
FILE				  *fptw_cor;				/* for saving to file corrected aIBIs */



void AddToBuffer(int	  aIBI,		  /* raw async (uncorrected) IBI to add to buffer */
				 int	  TimeStamp)  /* secondary timestamp for IBI (in milliseconds) */

{
static int	  LastTimeStamp;			/* timestamp from last aIBI (in ms) */
int			  i,j;
static int	  past_buffer[HISTORY_SIZE];  /* most recently corrected; used to compute statistics for thresholds */
static int	  past_count;				  /* count of IBIs in past_buffer (must get to 10 before going live) */
int			  buffer_delta_update;		  /* in applying ruleset, how many IBIs get accepted (could be 1-3) */
static int	  sequential_corrections;	  /* count of corrections made in a row */

		/* trigger to initialize buffer to empty */
if (aIBI == -1)
  {
  buffer_size=0;		/* no aIBI's in buffer yet */
  buffer_total_time=0;	/* no cumulative aIBI time in buffer yet */
  check_index=1;		/* have to assume first aIBI is ok - start checking at second aIBI */
  eat_index=1;			/* consumer will start eating at [1] when buffer is full enough */
  eat_delta=0;			/* portion of aIBI time already consumed at eat_index */
  LastTimeStamp=0;		/* timestamp of previous aIBI */
  past_count=0;			/* no history of corrected aIBI's yet */
  sequential_corrections=0;	/* no corrections made yet */
  if (Save_Cor_aIBI == 1)
	{
    if ((fptw_cor=fopen("C:\\Data\\cor_aIBI.txt","wb")) == NULL)
	  {
	  MessageBox(NULL,"Unable to open file for saving corrected aIBIs","Error",MB_OK | MB_APPLMODAL);
	  Save_Cor_aIBI=0;
	  }
	}
  return;
  }

		/* aIBI event triggered but value was garbled? */
if (aIBI < MIN_TRANSMIT_IBI  ||  aIBI > MAX_TRANSMIT_IBI)
  {
  if (TimeStamp != 0  &&		/* signifies no secondary timestamp available */
	  LastTimeStamp != 0  &&	/* can't use it for first aIBI */
	  TimeStamp-LastTimeStamp >= MIN_ACTUAL_IBI  &&  TimeStamp-LastTimeStamp <= MAX_ACTUAL_IBI)
	{	/* ignore aIBI; use difference in timestamps */
	ibi_buffer[buffer_size]=(TimeStamp-LastTimeStamp);
	buffer_size++;
	buffer_total_time+=(TimeStamp-LastTimeStamp);
	}
  else
	return;	  /* nothing we can do with this -- ignore it */
  }
else	/* we should be checking for (TimeStamp-LastTimeStamp) is too far from aIBI */
  {
  ibi_buffer[buffer_size]=aIBI;
  buffer_size++;
  buffer_total_time+=aIBI;
  }
LastTimeStamp=TimeStamp;

if (buffer_size >= BUFFER_SIZE)
  {
  MessageBox(NULL,"Buffer overfilled","Error - IBI_Buffer()",MB_OK | MB_APPLMODAL);
  exit(0);
  }

		/* can't check for errors until have at least 4 entries (context) */
if (buffer_size < 4)
  return;
		/* can't check for errors unless have at least 2 entries after check_index (in cases of combine3) */
if (buffer_size-check_index <= 2)
  return;

		/* we are checking buffer[i] using statistics of the past_buffer */
		/* the correction may affect any within [i]...[i+2], depending on if it combines */
		/* the acceptance test is to compare the correction against [i-1], which is the most recently accepted */
if (abs(ibi_buffer[check_index]-ibi_buffer[check_index-1]) > makeThreshold(past_buffer, past_count, 1)
				&&  sequential_corrections < 4  &&  UseCorrectionEngine == 1)
  {
		/* errors are checked in a certain order, because the first plausible fix found is applied */
  if (abs((ibi_buffer[check_index]/2)-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a double IBI -- [i] is the double -- split it */
	{
	for (i=buffer_size; i>check_index+1; i--)
	  ibi_buffer[i]=ibi_buffer[i-1];
	ibi_buffer[check_index]/=2;
	ibi_buffer[check_index+1]=ibi_buffer[check_index];
	buffer_size++;
	buffer_delta_update=2;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]/3)-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a triple IBI -- [i] is the triple -- divide it by three */
	{
	for (i=buffer_size+1; i>check_index+2; i--)
	  ibi_buffer[i]=ibi_buffer[i-2];
	ibi_buffer[check_index]/=3;
	ibi_buffer[check_index+2]=ibi_buffer[check_index+1]=ibi_buffer[check_index];
	buffer_size+=2;
	buffer_delta_update=3;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]+ibi_buffer[check_index+1])-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a split IBI -- [i] and [i+1] are the split -- combine them */
	{
	ibi_buffer[check_index]+=ibi_buffer[check_index+1];
	for (i=check_index+1; i<buffer_size-1; i++)
	  ibi_buffer[i]=ibi_buffer[i+1];
	buffer_size--;
	buffer_delta_update=1;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]+ibi_buffer[check_index+1])/2-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a split & combine IBI -- [i] and [i+1] are both in error -- average them */
	{
	ibi_buffer[check_index]=(ibi_buffer[check_index]+ibi_buffer[check_index+1])/2;
	ibi_buffer[check_index+1]=ibi_buffer[check_index];
	buffer_delta_update=2;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]+ibi_buffer[check_index+1]+ibi_buffer[check_index+2])/2-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a combine3/split2 -- [i]...[i+2] should be two IBIs, not three */
	{
	ibi_buffer[check_index]=(ibi_buffer[check_index]+ibi_buffer[check_index+1]+ibi_buffer[check_index+2])/2;
	ibi_buffer[check_index+1]=ibi_buffer[check_index];
	for (i=check_index+2; i<buffer_size-1; i++)
	  ibi_buffer[i]=ibi_buffer[i+1];
	buffer_size--;
	buffer_delta_update=2;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]+ibi_buffer[check_index+1])/3-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a combine2/split3 -- [i]...[i+1] should be three IBIs, not two */
	{
	for (i=buffer_size; i>check_index+2; i--)
	  ibi_buffer[i]=ibi_buffer[i-1];
	ibi_buffer[check_index]=(ibi_buffer[check_index]+ibi_buffer[check_index+1])/3;
	ibi_buffer[check_index+1]=ibi_buffer[check_index];
	ibi_buffer[check_index+2]=ibi_buffer[check_index];
	buffer_size++;
	buffer_delta_update=3;
	sequential_corrections++;
	}
  else if (abs((ibi_buffer[check_index]+ibi_buffer[check_index+1]+ibi_buffer[check_index+2])/3-ibi_buffer[check_index-1]) < makeThreshold(past_buffer, past_count, 2))
		/* looks like a combine3/split3 mess -- [i]...[i+2] need to be 3-way averaged (smoothed) */
	{
	ibi_buffer[check_index]=(ibi_buffer[check_index]+ibi_buffer[check_index+1]+ibi_buffer[check_index+2])/3;
	ibi_buffer[check_index+1]=ibi_buffer[check_index];
	ibi_buffer[check_index+2]=ibi_buffer[check_index];
	buffer_delta_update=3;
	sequential_corrections++;
	}
  else
		/* does not fit any known rule -- don't do anything? (filter?) (copy last?) */
	{
	// ibi_buffer[check_index]=ibi_buffer[check_index-1];
	buffer_delta_update=1;
	}
  }
else
		/* nothing suspicious looking about this IBI -- move check_index ahead */
  {
  buffer_delta_update=1;
  if (sequential_corrections > 0)
	sequential_corrections--;
  }

		/* move check_index ahead by how many got addressed during rule firing, above */
check_index+=buffer_delta_update;

		/* have to check this again, in case buffer grew */
if (buffer_size >= BUFFER_SIZE)
  {
  MessageBox(NULL,"Buffer overfilled","Error - IBI_Buffer()",MB_OK | MB_APPLMODAL);
  exit(0);
  }

		/* update the most recently corrected buffer (past_buffer) */
if (past_count < HISTORY_SIZE)
  {		/* building history of first HISTORY_SIZE corrected */
  for (i=0; i<buffer_delta_update; i++)
	{
    past_buffer[past_count]=ibi_buffer[check_index-buffer_delta_update+i];
    past_count++;
	}
  }
else
  {		/* toss out oldest, add on newest (just corrected), 1-3 times depending on rule just fired */
  for (j=0; j<buffer_delta_update; j++)
	{
    for (i=0; i<HISTORY_SIZE-1; i++)
	  past_buffer[i]=past_buffer[i+1];
    past_buffer[HISTORY_SIZE-1]=ibi_buffer[check_index-1];
	}
  }

}



		/*
		** Consumes a synchronous IBI from the buffer.  Returns 1 if successful.
		** Returns 0 if fails.  Actually, never fails now.  But assumes that the
		** buffer will be allowed to fill for 8 seconds before first access.
		**
		** Because the IBI-detector is potentially running on a different clock
		** (for example, on another computer), the "clock" of the cumulative aIBI's
		** can drift from the clock used in this program to sync analysis.
		** (An easy example to make the point is two desktop computers running side-by-side;
		** their desktop clocks can be off by minutes after less than one week.)
		** The synchronous consumer function (this one) handles this problem by
		** allowing "drift" between the two clock rates.  The buffer size is allowed
		** to drift between 6 and 10 seconds in length.  If the buffer shrinks to smaller
		** than 6 seconds then a consumption call will stall the clock for one tick
		** (in our case, 250 ms).  If the buffer expands to larger than 10 seconds then
		** the consumption call will accelerate the clock one extra tick.
		** In the absence of a master clock overseeing both processes, we cannot
		** think of a better strategy.  If both processes are running on the same
		** master clock, and coded appropriately, this problem should not occur.
		**
		** A second problem that could occur is the IBI-detector goes "nuts",
		** feeding the buffer at an impossibly fast rate, or at an impossibly slow rate.
		** (For example, the transmission medium breaks, or the detector shuts off.)
		** In this case we don't want to erroneously look at this as drift.
		** Therefore, if the buffer shrinks or grows in mad proportions, we should halt.
		*/

int GetSyncIBI(int *sIBI,int SampleTimeInMS)

{
int		i,j,update_ticks;

		/* return the IBI at the consumption index */
*sIBI=ibi_buffer[eat_index];

		/* check for drift to see how to update this "clock" */
if (buffer_total_time-SampleTimeInMS < 6000)
  update_ticks=0;
else if (buffer_total_time-SampleTimeInMS > 10000)
  update_ticks=2;
else
  update_ticks=1;

		/* move the aIBI "now"-reference forward */
for (j=0; j<update_ticks; j++)
  {
  eat_delta+=SampleTimeInMS;
		/* if we finished the current consumption index, eat into the next one (iteratively) */
  while (eat_delta >= ibi_buffer[eat_index]  &&  eat_index < buffer_size-1 /* just in case */)
	{
    eat_delta-=ibi_buffer[eat_index];
    eat_index++;
		/* update the buffer forward, dropping oldest aIBI */
	if (Save_Cor_aIBI == 1)
	  {
	  fprintf(fptw_cor,"%d\n",ibi_buffer[0]);
	  }
    for (i=0; i<buffer_size-1; i++)
	  ibi_buffer[i]=ibi_buffer[i+1];
    buffer_size--;
    eat_index--;
    check_index--;
	}
		/* update the cumulative aIBI clock held in the buffer */
  buffer_total_time-=SampleTimeInMS;
  }
return(1);
}



		/* Only used for display -- retrieve all buffer properties */

void DumpBuffer(int Buffer[],
				int *BufferSize,
				int *BufferTotalTime,
				int *CheckIndex,
				int *ConsumeIndex,
				int *ConsumeDelta)

{
int i;

for (i=0; i<buffer_size; i++)
  Buffer[i]=ibi_buffer[i];
*BufferSize=buffer_size;
*BufferTotalTime=buffer_total_time;
*CheckIndex=check_index;
*ConsumeIndex=eat_index;
*ConsumeDelta=eat_delta;
}





		/* Jeromie's function for determining the thresholds to use for
		** ED&C on an IBI.  Type=1 comptues the thresold used to detect
		** an error.  Type=2 computes the threshold used to accept a
		** suggested correction. */

int makeThreshold(int *past_buffer, int past_buffer_size, int type)

{
int i;
int msd; // mean succesive difference
int threshold;

		/* cannot compute reasonable statistics if have less than 10, so use default */
if (past_buffer_size < HISTORY_SIZE)
  return DEFAULT_THRESHOLD;
		/* error check - make sure type comes in reasonable value */
if (type != 1  &&  type != 2)
  return DEFAULT_THRESHOLD;
		/* compute the mean successive difference of the corrected portion of the buffer */
msd=0;
for(i=0; i<past_buffer_size-1; i++)
  msd += abs(past_buffer[i] - past_buffer[i+1]);
msd = msd / (past_buffer_size - 1);
		/* threshold is a constant multiplied by the MSD */
if (type == 1)
  threshold = msd * MULTIPLIER_1;
else 
  threshold = msd * MULTIPLIER_2;
		/* threshold is constrained to lie within a min...max range */
if (type == 1)
  {
  if (threshold < THRESH_MIN_DETECT)
	threshold = THRESH_MIN_DETECT;
  if (threshold > THRESH_MAX_DETECT)
	threshold = THRESH_MAX_DETECT;
  }
if (type == 2)
  {
  if (threshold < THRESH_MIN_ACCEPT)
	threshold = THRESH_MIN_ACCEPT;
  if (threshold > THRESH_MAX_ACCEPT)
	threshold = THRESH_MAX_ACCEPT;
  }
return threshold;
}


