
/* See file "monitor.c" for copyright and other information. */

#include <windows.h>
#include <time.h>

#define SQR(x) ((x)*(x))	/* macro for square */
#ifndef M_PI			/* in case M_PI not found in math.h */
#define M_PI 3.1415927
#endif
#ifndef M_E
#define M_E 2.718282
#endif

#define MAX_FILENAME_CHARS	200

char	CurrentPath[MAX_FILENAME_CHARS];
char	ParamsFilename[MAX_FILENAME_CHARS];

#define VersionText	"2.5"

HWND	MainWnd;

		// for accessing shared memory space for sending ameter readings to other processes
HANDLE	SharedMemoryHandle;

		// for passing asynchronous raw IBIs between IBI detector thread and monitor/analysis thread
int		new_aIBI_value;
int		new_aIBI_timestamp;
int		new_aIBI_flag;

		// Program control flags
int		MonitorOn;				/* main analysis thread (Monitor) is running? */
int		AnalysisMode;			/* =0 live signal */
								/* =1 test signal */
								/* =2 playback */
int		DisplayMode;			/* =0 no display */
								/* =1 normal display (details) */
								/* =2 gauge display */
								/* =3 baloon display */
								/* =4 streaming display */
int		Save_Cor_aIBI;			/* save (to file) corrected aIBIs? */
int		CalibrationLocked;		/* 0=>continuously recalibrate; 1=>use constant values from input */

		// Display variables
#define	TOP_WINDOW_MARGIN	50
#define SIDE_WINDOW_MARGIN	70
#define TEXT_DISPLAY_WIDTH	240
#define TICKMARK_THICKNESS	10
//#define	ZERO_BPM			2045  /* units are mV (A/D reads 2050 mV at zero beats per minute) */
//#define	DELTA_BPM			8.167 /* units are mV per BPM */
//#define	ZERO_MPB			2042  /* units are mV (A/D reads 2042 mV at zero msec per beat) */
//#define	DELTA_MPB			0.82  /* units are mV per (msec per beat) */
int		PlotDataMinValue,PlotDataMaxValue;	/* clipping range for displaying raw data */
int		PlotDataRange;						/* X-axis range of time to plot raw data (seconds) */
int		PowerDataMinValue,PowerDataMaxValue;/* clipping range for displaying power data */
int		PowerDataRange;						/* x-axis range of CPM to display power data */
int		RSADataRange;						/* x-axis range of time to plot RSA (seconds) */
int		RSALongDataRange;					/* x-axis range of time to plot long RSA (minutes) */
int		NormDistMinValue,NormDistMaxValue;	/* sampling range to display dist of arousal */
int		HeartRateWindowRows,HeartRateWindowCols;
int		PowerWindowRows,PowerWindowCols;
int		RSAWindowRows,RSAWindowCols;
int		RSALongWindowRows,RSALongWindowCols;
int		NormWindowRows,NormWindowCols;
#define GAUGE_RADIUS		150

		// Analysis variables
#define MAX_SAMPLING_RATE	100		/* 100 Hz max sampling rate for synchronous IBIs */
#define MAX_SAMPLING_WINDOW	64		/* 64 seconds max window over which to analyze IBIs */
#define MAX_DATA			(MAX_SAMPLING_RATE*MAX_SAMPLING_WINDOW)	/* buffer size for IBI data */
#define TOTAL_FREQS			30		/* analysis always done for these frequencies */
int		SamplingRate;				/* units are samples per second (Hz) */
int		WindowSec;					/* units are seconds */
int		WindowSize;					/* units are number of samples */
double	ArousalSmooth;				/* unitless smoothing factor used in Kalman filter */
double	ZScoreRange;				/* units are std devs, range of interest in Ameter gauge */
double	MeanRSA,StdDevRSA;			/* statistics of RSA over all time */
int		Minutes;					/* so user can calibrate RSA statistics */
int		TotalMinutes;				/* for displaying RSA statistics */
int		RSALongPlot[1000];			/* for maintaining history in long RSA window */
int		NormHist[1000];				/* histogram of arousal */
int		UseTimeStamps;				/* variable for error detect/correct (playback only) */
int		UseCorrectionEngine;		/* detect/correct IBI errors, or not?  (1/0) */

		// Playback variables
int		PlaybackSpeed;		/* 1 = 1x, 2 = 2x, etc. */
char	PlaybackFilename[MAX_FILENAME_CHARS]; /* does not include path */

		// Function prototypes
LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);
void PaintImage();
LRESULT CALLBACK AnalysisParamsProc(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK DisplayParamsProc(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK PopParamsProc(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK PlaybackParamsProc(HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK OpenFileProc(HWND,UINT,WPARAM,LPARAM);

void Display_aIBI(HWND,int,int);
void DisplayResult(HWND,int,int,int,int,int,double *,
				   int,double,double,double,double,int);
void DisplayHeartrate(HWND,int,int,int,int,int,int,int,int);
void DisplayRSA(HWND,int,int,int,int,double,int,int,int);
void DisplayGauge(HWND,int,int,double,int);

void Monitor(void *);

void PowerAnalyze(short *,double *,int,int);
int ArousalAnalysis(int,int,int,double,int *,double *,double *,int *,double *,
					 double *,int *,double *);

void AddToBuffer(int,int);
int GetSyncIBI(int *,int);
void DumpBuffer(int *,int *,int *,int *,int *,int *);
int makeThreshold(int *,int,int);

int IBI_TestSignal(int,int);
void SetUpPlayback();
int CheckPlayback(unsigned long int);
void ClosePlayback();

void EZIBI(void *);

