
		/*
		*/

#include <stdio.h>
#include <math.h>		/* for sin/cos functions in IBI_TestSignal() */
#include <windows.h>	/* for MessageBox() */
#include <time.h>		/* for clock_t type and clock() function */
#include <winbase.h>	/* for serial port functions */
//#include <cbw.h>		/* PCM-DAS08 A/D card functions */
#include "globals.h"


int IBI_TestSignal(int Time,
				   int SamplingRateInMS)

{
static int			  DataValue;			/* current IBI being counted off */
int					  NextDataValue;		/* used when current IBI fires to calc next one */
int					  LocalTimeStamp;
int					  SimulateErrors=0;		/* turns on/off simulated errors */
int					  ErrorCounter=0;		/* cycles through different error types */
static int			  TimeRemainingInCurrentIBI;

if (Time == 0)
  {		/* first IBI will occur in ... */
  DataValue=(int)(650.0+50.0*sin((double)Time/60000.0*3.5*(2.0*M_PI))
					   +50.0*sin((double)Time/60000.0*16.0*(2.0*M_PI)));
  TimeRemainingInCurrentIBI=DataValue;
  return(0);
  }
		/* counting off until upcoming IBI fires */
TimeRemainingInCurrentIBI-=SamplingRateInMS;
if (TimeRemainingInCurrentIBI > 0)
  return(0);

		/* for testing purposes, this will inject various types of errors */
if (ErrorCounter == 10)
  {	/* simulate split error (first half = 40%) */
  DataValue=(unsigned short)(0.4*(650.0+50.0*sin(Time/60000.0*3.5*(2.0*M_PI))+50.0*sin(Time/60000.0*16.0*(2.0*M_PI))));
  LocalTimeStamp=Time; /* timestamp the IBI locally */
  }
else if (ErrorCounter == 11)
  {	/* simulate split error (second half = 60%) */
  DataValue=(unsigned short)(0.6*(650.0+50.0*sin(Time/60000.0*3.5*(2.0*M_PI))+50.0*sin(Time/60000.0*16.0*(2.0*M_PI))));
  LocalTimeStamp=Time; /* timestamp the IBI locally */
  }
else if (ErrorCounter == 19)
  {	/* simulate missed error (double aIBI) */
  DataValue=(unsigned short)(2.0*(650.0+50.0*sin(Time/60000.0*3.5*(2.0*M_PI))+50.0*sin(Time/60000.0*16.0*(2.0*M_PI))));
  LocalTimeStamp=Time; /* timestamp the IBI locally */
  }
else if (ErrorCounter == 25)
  {	/* simulate garbled transmission error (aIBI wrong, but timestamp ok) */
  DataValue=(unsigned short)(1.0*(650.0+50.0*sin(Time/60000.0*3.5*(2.0*M_PI))+50.0*sin(Time/60000.0*16.0*(2.0*M_PI))));
  DataValue=(unsigned short)3500;
  LocalTimeStamp=Time; /* timestamp the IBI locally */
  }
else
  {	/* no error - good aIBI */
  NextDataValue=(unsigned short)(650.0+50.0*sin((double)Time/60000.0*3.5*(2.0*M_PI))
									  +50.0*sin((double)Time/60000.0*16.0*(2.0*M_PI)));
  LocalTimeStamp=Time+TimeRemainingInCurrentIBI;
  }
if (SimulateErrors == 1)
  ErrorCounter=(ErrorCounter+1)%30;
  
new_aIBI_value=DataValue;
new_aIBI_timestamp=LocalTimeStamp;
new_aIBI_flag=1;
DataValue=NextDataValue;
TimeRemainingInCurrentIBI+=DataValue;
return(1);
}



void EZIBI(void *dummy)

{
HANDLE			hComm;
DCB				CommSettings;
COMMTIMEOUTS	CommTimeoutSettings;
unsigned char	SerialPortBuffer[10];
int				BytesRead,i;
unsigned short	DataValue;
DWORD			EventType;

		/* open serial port and configure to talk to EZIBI device */
hComm=CreateFile("COM1",GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
if (hComm == INVALID_HANDLE_VALUE)
  {
  MessageBox(NULL,"Unable to connect to serial port COM1","Error",MB_OK | MB_APPLMODAL);
  return;
  }
GetCommState(hComm,&CommSettings);
CommSettings.BaudRate=38400;
CommSettings.fParity=0;
CommSettings.Parity=0;
CommSettings.ByteSize=8;
CommSettings.StopBits=0;
SetCommState(hComm,&CommSettings);
GetCommTimeouts(hComm,&CommTimeoutSettings);
CommTimeoutSettings.ReadIntervalTimeout=0; // turn these timeouts off, or up to 20 or higher
CommTimeoutSettings.ReadTotalTimeoutConstant=0;
CommTimeoutSettings.ReadTotalTimeoutMultiplier=0;
SetCommTimeouts(hComm,&CommTimeoutSettings);	/* no timeouts on serial port read */
SetCommMask(hComm,EV_RXCHAR);	/* set mask to fire when byte received */

while (MonitorOn)
  {
	  /* wait until first byte arrives, then read 3 (with 40ms timeout, which was set above) */
  WaitCommEvent(hComm,&EventType,NULL);
	  /* get data from serial port */
  BytesRead=0;
  while (BytesRead != 3)
    i=ReadFile(hComm,SerialPortBuffer,3,&BytesRead,NULL);
  DataValue=((unsigned short)(SerialPortBuffer[1] & 0x0F) << 8)+(unsigned short)(SerialPortBuffer[2]);
  new_aIBI_value=DataValue;
  new_aIBI_timestamp=(int)((long int)clock()*(long int)1000/(long int)CLOCKS_PER_SEC); /* timestamp the IBI locally */
  new_aIBI_flag=1;
  }

CloseHandle(hComm);
}


