
/* See file "monitor.c" for copyright and other information. */

#include <stdio.h>
#include <math.h>	  /* for sin/cos function for gauge display */
#include <windows.h>
#include <winuser.h>
#include "resource.h"
#include "globals.h"



LRESULT CALLBACK AnalysisParamsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
BOOL		Success;
int			i;
char		text[80];

switch(uMsg)
  {
  case WM_INITDIALOG:
	SetDlgItemInt(hDlg,ID_PARAM_1,SamplingRate,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_2,WindowSec,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_3,WindowSize,FALSE);
	sprintf(text,"%lf",ArousalSmooth);
	SetDlgItemText(hDlg,ID_PARAM_4,text);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_PARAM_1:
		SamplingRate=GetDlgItemInt(hDlg,ID_PARAM_1,&Success,FALSE);
		if (SamplingRate < 1) SamplingRate=1;
		if (SamplingRate > MAX_SAMPLING_RATE) SamplingRate=MAX_SAMPLING_RATE;
		break;
	  case ID_PARAM_2:
		WindowSec=GetDlgItemInt(hDlg,ID_PARAM_2,&Success,FALSE);
		if (WindowSec < 15) WindowSec=15;
		if (WindowSec > MAX_SAMPLING_WINDOW) WindowSec=MAX_SAMPLING_WINDOW;
		break;
	  case ID_PARAM_3:
		break;
	  case ID_PARAM_4:
		GetDlgItemText(hDlg,ID_PARAM_4,text,79);
		ArousalSmooth=atof(text);
		if (ArousalSmooth < 0) ArousalSmooth=0;
		if (ArousalSmooth > 1000000) ArousalSmooth=1000000;
		break;
	  case IDOK:
		EndDialog(hDlg,IDOK);
		break;
	  }
	WindowSize=WindowSec*SamplingRate;
	i=1; while (i < WindowSize) i*=2;
	WindowSize=i; WindowSec=WindowSize/SamplingRate;
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}




LRESULT CALLBACK DisplayParamsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
BOOL		Success;
char		text[80];
int			mode,id;
HWND		hControl;

switch(uMsg)
  {
  case WM_INITDIALOG:
	SetDlgItemInt(hDlg,ID_PARAM_1,PlotDataMinValue,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_2,PlotDataMaxValue,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_3,PlotDataRange,FALSE);
	sprintf(text,"%3.1lf",ZScoreRange);
	SetDlgItemText(hDlg,ID_PARAM_4,text);
	SetDlgItemInt(hDlg,ID_PARAM_5,PowerDataMinValue,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_6,PowerDataMaxValue,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_7,RSADataRange,FALSE);
	SetDlgItemInt(hDlg,ID_PARAM_8,RSALongDataRange,FALSE);
	if (DisplayMode == 0)
	  CheckRadioButton(hDlg,IDD_RADIO1,IDD_RADIO5,IDD_RADIO2);
	else if (DisplayMode == 2)
	  CheckRadioButton(hDlg,IDD_RADIO1,IDD_RADIO5,IDD_RADIO3);
	else if (DisplayMode == 3)
	  CheckRadioButton(hDlg,IDD_RADIO1,IDD_RADIO5,IDD_RADIO4);
	else if (DisplayMode == 4)
	  CheckRadioButton(hDlg,IDD_RADIO1,IDD_RADIO5,IDD_RADIO5);
	else
	  {
	  CheckRadioButton(hDlg,IDD_RADIO1,IDD_RADIO5,IDD_RADIO1);
	  DisplayMode=1;  /* redundant - make sure display mode has good value */
	  }
	if (DisplayMode == 1)
	  mode=TRUE;
	else
	  mode=FALSE;
	for (id=ID_PARAM_1; id<=ID_PARAM_8; id++)
	  {
	  hControl=GetDlgItem(hDlg,id);
	  EnableWindow(hControl,mode);
	  }
	hControl=GetDlgItem(hDlg,ID_PARAM_4);
	EnableWindow(hControl,TRUE);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_PARAM_1:	/* heartrate (raw) data min */
		PlotDataMinValue=GetDlgItemInt(hDlg,ID_PARAM_1,&Success,FALSE);
		if (PlotDataMinValue < 0) PlotDataMinValue=0;
		if (PlotDataMinValue > 800) PlotDataMinValue=800;
		break;
	  case ID_PARAM_2:	/* heartrate (raw) data max */
		PlotDataMaxValue=GetDlgItemInt(hDlg,ID_PARAM_2,&Success,FALSE);
		if (PlotDataMaxValue < 1000) PlotDataMaxValue=1000;
		if (PlotDataMaxValue > 2000) PlotDataMaxValue=2000;
		break;
	  case ID_PARAM_3:	/* heartrate (raw) data range (X-axis) */
		PlotDataRange=GetDlgItemInt(hDlg,ID_PARAM_3,&Success,FALSE);
		if (PlotDataRange < 5) PlotDataRange=5;
		if (PlotDataRange > 300) PlotDataRange=300;
		break;
	  case ID_PARAM_4:	/* Z score range (scaling Ameter gauge) */
		GetDlgItemText(hDlg,ID_PARAM_4,text,79);
		ZScoreRange=atof(text);
		if (ZScoreRange < 0.5) ZScoreRange=0.5;
		if (ZScoreRange > 5.0) ZScoreRange=5.0;
		break;
	  case ID_PARAM_5:	/* power data min */
		PowerDataMinValue=GetDlgItemInt(hDlg,ID_PARAM_5,&Success,FALSE);
		if (PowerDataMinValue < 0) PowerDataMinValue=0;
		if (PowerDataMinValue > 0) PowerDataMinValue=0;
		break;
	  case ID_PARAM_6:	/* power data max */
		PowerDataMaxValue=GetDlgItemInt(hDlg,ID_PARAM_6,&Success,FALSE);
		if (PowerDataMaxValue < 100) PowerDataMaxValue=100;
		if (PowerDataMaxValue > 20000) PowerDataMaxValue=20000;
		break;
	  case ID_PARAM_7:	/* RSA data range (X-axis) */
		RSADataRange=GetDlgItemInt(hDlg,ID_PARAM_7,&Success,FALSE);
		if (RSADataRange < 5) RSADataRange=5;
		if (RSADataRange > 300) RSADataRange=300;
		break;
	  case ID_PARAM_8:	/* RSA long data range (X-axis) */
		RSALongDataRange=GetDlgItemInt(hDlg,ID_PARAM_8,&Success,FALSE);
		if (RSALongDataRange < 5) RSALongDataRange=5;
		if (RSALongDataRange > 300) RSALongDataRange=300;
		break;
	  case IDD_RADIO1:
		DisplayMode=1;
		for (id=ID_PARAM_1; id<=ID_PARAM_8; id++)
		  {
		  hControl=GetDlgItem(hDlg,id);
		  EnableWindow(hControl,TRUE);
		  }
		PaintImage();
		break;
	  case IDD_RADIO2:case IDD_RADIO3:case IDD_RADIO4:case IDD_RADIO5:
		if (LOWORD(wParam) == IDD_RADIO2)
		  DisplayMode=0;
		else if (LOWORD(wParam) == IDD_RADIO3)
		  DisplayMode=2;
		else if (LOWORD(wParam) == IDD_RADIO4)
		  DisplayMode=3;
		else if (LOWORD(wParam) == IDD_RADIO5)
		  DisplayMode=4;
		for (id=ID_PARAM_1; id<=ID_PARAM_8; id++)
		  {
		  hControl=GetDlgItem(hDlg,id);
		  if ((DisplayMode == 2  &&  id == ID_PARAM_4)  ||
			  (DisplayMode == 3  &&  id == ID_PARAM_4)  ||
			  (DisplayMode == 4  &&  (id == ID_PARAM_1  ||  id == ID_PARAM_2  ||  id == ID_PARAM_3)))
			EnableWindow(hControl,TRUE);
		  else
		    EnableWindow(hControl,FALSE);
		  }
		PaintImage();
		break;
	  case IDOK:
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}



LRESULT CALLBACK PopParamsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
BOOL		Success;
char		text[80];
static int	LastLockValue;

switch(uMsg)
  {
  case WM_INITDIALOG:
	SetDlgItemInt(hDlg,ID_PARAM_1,Minutes,FALSE);
	sprintf(text,"%lf",MeanRSA);
	SetDlgItemText(hDlg,ID_PARAM_2,text);
	sprintf(text,"%lf",StdDevRSA);
	SetDlgItemText(hDlg,ID_PARAM_3,text);
	if (CalibrationLocked == 0)
	  CheckDlgButton(hDlg,IDC_LOCK_RADIO,BST_UNCHECKED);
	else
	  CheckDlgButton(hDlg,IDC_LOCK_RADIO,BST_CHECKED);
	LastLockValue=CalibrationLocked;
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case ID_PARAM_1:
		Minutes=GetDlgItemInt(hDlg,ID_PARAM_1,&Success,FALSE);
		if (Minutes < 0) Minutes=0;
		if (Minutes > 100000) Minutes=100000;
		break;
	  case ID_PARAM_2:
		GetDlgItemText(hDlg,ID_PARAM_2,text,79);
		MeanRSA=atof(text);
		if (MeanRSA < 0.0) MeanRSA=0.0;
		if (MeanRSA > 100000) MeanRSA=100000;
		break;
	  case ID_PARAM_3:
		GetDlgItemText(hDlg,ID_PARAM_3,text,79);
		StdDevRSA=atof(text);
		if (StdDevRSA < 0.0) StdDevRSA=0.0;
		if (StdDevRSA > 100000) StdDevRSA=100000;
		break;
	  case IDC_LOCK_RADIO:
		PostMessage(MainWnd,WM_COMMAND,ID_LOCK_CALIB,0);
		LastLockValue=(LastLockValue+1)%2;
		if (LastLockValue == 0)
		  CheckDlgButton(hDlg,IDC_LOCK_RADIO,BST_UNCHECKED);
		else
		  CheckDlgButton(hDlg,IDC_LOCK_RADIO,BST_CHECKED);
		break;
	  case IDOK:
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}



LRESULT CALLBACK PlaybackParamsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)

{
switch(uMsg)
  {
  case WM_INITDIALOG:
	if (PlaybackSpeed == 1)
	  CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO1);
	else if (PlaybackSpeed == 2)
	  CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO2);
	else if (PlaybackSpeed == 4)
	  CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO3);
	else if (PlaybackSpeed == 10)
	  CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO4);
	else
	  CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO5);
	break;
  case WM_COMMAND:
	switch(LOWORD(wParam))
	  {
	  case IDC_RADIO1:
		PlaybackSpeed=1;
	    CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO1);
		break;
	  case IDC_RADIO2:
		PlaybackSpeed=2;
	    CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO2);
		break;
	  case IDC_RADIO3:
		PlaybackSpeed=4;
	    CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO3);
		break;
	  case IDC_RADIO4:
		PlaybackSpeed=10;
	    CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO4);
		break;
	  case IDC_RADIO5:
		PlaybackSpeed=999999;	/* suitable large number to serve as divider */
	    CheckRadioButton(hDlg,IDC_RADIO1,IDC_RADIO6,IDC_RADIO5);
		break;
	  case IDOK:
		EndDialog(hDlg,IDOK);
		break;
	  }
	break;
  default:
	return(FALSE);
  }
return(TRUE);
}






void Display_aIBI(HWND		  hWnd,					/* display window */
				  int		  aIBI,					/* current async IBI displayed in text */
				  int		  WarningOnSamplingRate)/* ms to display warning about timing off */

{
HDC				hDC;
char			text[300],text2[300];
RECT			TextRect;
HBRUSH			DrawBrush;
int				x,y,i;
SIZE			TextSize;
int				Buffer[128],BufferSize,BufferTotalTime,CheckIndex,ConsumeIndex,ConsumeDelta;

		/* display raw data reading */
hDC=GetDC(hWnd);
y=TextRect.top=10;
TextRect.bottom=y+TOP_WINDOW_MARGIN/2;
x=TextRect.left=10;
TextRect.right=TEXT_DISPLAY_WIDTH-10;
if (WarningOnSamplingRate > 0)
  DrawBrush=CreateSolidBrush(RGB(255,235,2)); /* yellow, warn user about missed sample(s) */
else
  DrawBrush=CreateSolidBrush(RGB(255,235,200)); /* pinkish, sampling rate is okay */
FillRect(hDC,&TextRect,DrawBrush);
DeleteObject(DrawBrush);
sprintf(text,"Raw data value: %d",aIBI);
GetTextExtentPoint32(hDC,text,strlen(text),&TextSize);
TextOut(hDC,TEXT_DISPLAY_WIDTH/2-TextSize.cx/2,y+TOP_WINDOW_MARGIN/4-TextSize.cy/2,text,strlen(text));
ReleaseDC(hWnd,hDC);

		/* display IBI buffer and properties */
if (DisplayMode == 1)
  {
  DumpBuffer(Buffer,&BufferSize,&BufferTotalTime,&CheckIndex,&ConsumeIndex,&ConsumeDelta);
  Buffer[BufferSize]=aIBI;
  BufferSize++;
  x=10;
  y=TOP_WINDOW_MARGIN;
  sprintf(text,"%d",Buffer[0]);
  for (i=1; i<BufferSize; i++)
	{
	sprintf(text2," %d",Buffer[i]);
	strcat(text,text2);
	}
  strcat(text,"                              ");
  hDC=GetDC(hWnd);
  TextOut(hDC,300,10,text,strlen(text));
  sprintf(text,"Buffer Time:  %d   ",BufferTotalTime);
  TextOut(hDC,x,y+220,text,strlen(text));
  sprintf(text,"CheckIndex:  %d   ",CheckIndex);
  TextOut(hDC,x,y+240,text,strlen(text));
  sprintf(text,"ConsumeIndex:  %d   ",ConsumeIndex);
  TextOut(hDC,x,y+260,text,strlen(text));
  sprintf(text,"ConsumeDelta:  %d   ",ConsumeDelta);
  TextOut(hDC,x,y+280,text,strlen(text));
  ReleaseDC(hWnd,hDC);
  }

}




void DisplayResult(HWND		  hWnd,					/* display window */
				   int		  DisplayMode,			/* determines what parts are displayed */
				   int		  sIBI,					/* current sync IBI displayed in heartrate window */
				   int		  LocalSamplingRate,	/* used to scale X-axes of heartrate & RSA */
				   int		  Time,					/* used to scale X-axis of heartrate & RSA */
				   int		  SamplesNeeded,		/* number of IBIs before power goes live */
				   double	  Power[TOTAL_FREQS],	/* current power spectrum result */
				   int		  PowerSetsNeeded,		/* number of IBIs before RSA goes live */
				   double	  Arousal,				/* current arousal */
				   double	  MeanRSA,				/* mean RSA over all time */
				   double	  StdDevRSA,			/* std dev RSA over all time */
				   double	  CurrentRSA,			/* current RSA (max power bar) */
				   int		  TotalMinutes)			/* total minutes arousal has been live */
{
HDC				hDC;
char			text[300];
int				x,y,i,Max,BarLevel;
RECT			TextRect,WindowRect;
HBRUSH			DrawBrush,OrigBrush;
HPEN			DrawPen,OrigPen;
SIZE			TextSize;
static int		AccumRSACount,AccumRSATime,RSALongResolution;
static double	AccumRSA;
static double	HeartWindowX;
static int		RSAWindowX,RSALongPlotX;
double			BarPower,GaugeReading,NormZ;
static int		last_radius;
int				radius;

		/* initialize display */
if (sIBI == -1)
  {
		/* initialize x-axis location of time series plots */
  HeartWindowX=0.0;
  RSAWindowX=0;
  RSALongPlotX=0;
		/* initialize RSA long plot values to -1, signifying don't draw anything */
  for (i=0; i<RSALongWindowCols; i++)
    RSALongPlot[i]=-1;
		/* initialize RSA scalars for plotting */
  AccumRSA=AccumRSACount=AccumRSATime=0;
		/* initialize graphics values for non-gauge displays */
  last_radius=0;
  return;
  }

if (DisplayMode == 1)
  {
  DisplayHeartrate(hWnd,TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN,TOP_WINDOW_MARGIN,(int)HeartWindowX,1,sIBI,
					LocalSamplingRate,HeartRateWindowRows,HeartRateWindowCols);

		/* update X-axis display location */
  HeartWindowX+=(((double)HeartRateWindowCols/(double)PlotDataRange)/(double)LocalSamplingRate);
  if ((int)HeartWindowX/* +(TextRect.right-TextRect.left)*/ > HeartRateWindowCols)
	HeartWindowX=0.0;

		/* erase power window */
  hDC=GetDC(hWnd);
  y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN;
  TextRect.bottom=y+PowerWindowRows;
  x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols/2-PowerWindowCols/2;
  TextRect.right=x+PowerWindowCols;
  DrawBrush=CreateSolidBrush(RGB(200,200,200));
  FillRect(hDC,&TextRect,DrawBrush);
  DeleteObject(DrawBrush);
  ReleaseDC(hWnd,hDC);
		/* if not enough data in buffer for power analysis, report how many more needed */
  if (SamplesNeeded)
	{
	hDC=GetDC(hWnd);
	sprintf(text,"N - %d",SamplesNeeded);
	GetTextExtentPoint32(hDC,text,strlen(text),&TextSize);
	TextOut(hDC,x+PowerWindowCols/2-TextSize.cx/2,y+PowerWindowRows/2-TextSize.cy/2,text,strlen(text));
	ReleaseDC(hWnd,hDC);
	}
  else	/* draw current power spectrum result */
	{
	hDC=GetDC(hWnd);
	DrawBrush=CreateSolidBrush(RGB(0,0,255));
	for (i=0; i<PowerDataRange; i++)
	  {
	  if (Power[i] < PowerDataMinValue) BarPower=PowerDataMinValue;
	  else if (Power[i] > PowerDataMaxValue) BarPower=PowerDataMaxValue;
	  else BarPower=Power[i];
	  y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows-
				  (int)((double)(BarPower-PowerDataMinValue)/
				  (double)(PowerDataMaxValue-PowerDataMinValue)*(double)PowerWindowRows)-
				  TICKMARK_THICKNESS/2;
	  TextRect.bottom=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows;
	  x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols/2-PowerWindowCols/2+
						(i*(PowerWindowCols/PowerDataRange));
	  TextRect.right=x+(PowerWindowCols/PowerDataRange);
				/* check for clipping on window boundaries */
	  if (TextRect.right == TextRect.left)
		TextRect.right=TextRect.left+1;
	  if (TextRect.right > TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols/2+PowerWindowCols/2)
		TextRect.right=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols/2+PowerWindowCols/2;
	  if (TextRect.top < TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN)
		TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN;
	  if (TextRect.bottom > TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows)
		TextRect.bottom=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows;
	  FillRect(hDC,&TextRect,DrawBrush);
	  }
	DeleteObject(DrawBrush);
	ReleaseDC(hWnd,hDC);
	}

		/* if not enough power results yet, report how many more needed */
  if (PowerSetsNeeded > 0)
	{
	hDC=GetDC(hWnd);
	y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN;
	TextRect.bottom=y+RSAWindowRows;
	x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN;
	TextRect.right=x+RSAWindowCols;
	DrawBrush=CreateSolidBrush(RGB(200,200,200));
	FillRect(hDC,&TextRect,DrawBrush);
	DeleteObject(DrawBrush);
	if (PowerSetsNeeded > 1)
	  {
	  sprintf(text,"N - %d",PowerSetsNeeded);
	  GetTextExtentPoint32(hDC,text,strlen(text),&TextSize);
	  TextOut(hDC,x+RSAWindowCols/2-TextSize.cx/2,y+RSAWindowRows/2-TextSize.cy/2,text,strlen(text));
	  ReleaseDC(hWnd,hDC);
	  }
	}
  if (PowerSetsNeeded <= 0)	/* display short-term and long-term RSA readings, and gauge reading */
	{
		  /* add RSA reading to long-term average RSA reading */
	AccumRSA+=CurrentRSA;
	AccumRSACount++;
	AccumRSATime+=1000/LocalSamplingRate;
	RSALongResolution=(RSALongDataRange*60*1000)/RSALongWindowCols;	/* units are MS per pixel */
	if (AccumRSATime >= RSALongResolution)
	  {
	  RSALongPlot[RSALongPlotX]=(int)(AccumRSA/(double)AccumRSACount+0.5);
	  RSALongPlotX=(RSALongPlotX+1)%RSALongWindowCols;
	  for (i=0; i<30; i++)	  /* clear space in front of current time */
		RSALongPlot[(RSALongPlotX+i)%RSALongWindowCols]=-1;	/* -1 means not valid - don't draw */
	  AccumRSA=0.0;
	  AccumRSACount=0;
	  AccumRSATime=0;
	  }

    DisplayRSA(hWnd,TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN,
					TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN,
					(int)RSAWindowX,0,CurrentRSA,
					LocalSamplingRate,RSAWindowRows,RSAWindowCols);

		/* update X-axis display location */
	RSAWindowX=(int)((double)((int)Time%(RSADataRange*1000))*
				  ((double)RSAWindowCols/(double)(RSADataRange*1000)));
	if (RSAWindowX /* +(TextRect.right-TextRect.left) */ > RSAWindowCols)
	  RSAWindowX=0;

		/* plot all RSA long data */
	hDC=GetDC(hWnd);
	for (i=0; i<RSALongWindowCols; i++)
	  {
	  if (RSALongPlot[i] == -1)
 		{
		y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+RSAWindowRows+TOP_WINDOW_MARGIN;
		TextRect.bottom=y+RSALongWindowRows;
		x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+i;
		TextRect.right=x+1;
		DrawBrush=CreateSolidBrush(RGB(200,200,200));
		FillRect(hDC,&TextRect,DrawBrush);
		DeleteObject(DrawBrush);
		continue;
		}
	  y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+RSAWindowRows+TOP_WINDOW_MARGIN+RSALongWindowRows
				  -(int)((double)(RSALongPlot[i]-PowerDataMinValue)/
				  (double)(PowerDataMaxValue-PowerDataMinValue)*(double)RSALongWindowRows)-
				  TICKMARK_THICKNESS/2;
	  TextRect.bottom=y+TICKMARK_THICKNESS;
	  x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+i;
	  TextRect.right=x+1;
				 /* check for clipping on window boundaries */
	  if (TextRect.right == TextRect.left) TextRect.right=TextRect.left+1;
	  if (TextRect.right > TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+RSALongWindowCols)
		TextRect.right=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+RSALongWindowCols;
	  if (TextRect.top < TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows+TOP_WINDOW_MARGIN)
		{
		TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows+TOP_WINDOW_MARGIN;
		TextRect.bottom=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows+TOP_WINDOW_MARGIN+TICKMARK_THICKNESS/2;
		}
	  if (TextRect.bottom > TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+RSAWindowRows+TOP_WINDOW_MARGIN+RSALongWindowRows)
		{
		TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+RSAWindowRows+TOP_WINDOW_MARGIN+RSALongWindowRows-TICKMARK_THICKNESS/2;
		TextRect.bottom=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+RSAWindowRows+TOP_WINDOW_MARGIN+RSALongWindowRows;
		}
	  DrawBrush=CreateSolidBrush(RGB(100,200,100));
	  FillRect(hDC,&TextRect,DrawBrush);
	  DeleteObject(DrawBrush);
	  }
	ReleaseDC(hWnd,hDC);

			  /* display the log normal distribution of arousal */
	x=TextRect.left=10;
	y=TextRect.top=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows+TOP_WINDOW_MARGIN;
	TextRect.right=x+NormWindowCols;
	TextRect.bottom=y+NormWindowRows;
	DrawBrush=CreateSolidBrush(RGB(200,200,200));
	FillRect(hDC,&TextRect,DrawBrush);
	DeleteObject(DrawBrush);
	hDC=GetDC(hWnd);
	DrawBrush=CreateSolidBrush(RGB(0,255,0));
	Max=0;
	for (i=1; i<1000; i++)
	  if (NormHist[i] > NormHist[Max])
		Max=i;
	x=10;
	y=TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN+PowerWindowRows+TOP_WINDOW_MARGIN;
	for (i=0; i<1000; i++)
	  {
	  if (NormHist[i] == 0)
		BarLevel=0;
	  else
		BarLevel=(int)(((double)NormHist[i]/(double)NormHist[Max])*(double)NormWindowRows);
	  TextRect.left=x+(int)((double)i/1000.0*(double)NormWindowCols);
	  TextRect.right=TextRect.left+1;
	  TextRect.bottom=y+NormWindowRows;
	  TextRect.top=TextRect.bottom-BarLevel;
	  FillRect(hDC,&TextRect,DrawBrush);
	  }
	DeleteObject(DrawBrush);
	ReleaseDC(hWnd,hDC);

			  /* display gauge reading */
	DisplayGauge(hWnd,TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN+RSAWindowCols/2,
				  TOP_WINDOW_MARGIN+HeartRateWindowRows,Arousal,GAUGE_RADIUS);

	}		/* end if (RSA is live) */
  }		/* end if (DisplayMode == 1) */

		/* display timer and RSA statistics */
if (DisplayMode == 1  ||  (DisplayMode == 0  &&  (PlaybackSpeed <= 10  ||  (int)Time%10000 == 0)))
  {
  hDC=GetDC(hWnd);
  y=TextRect.top=60;
  TextRect.bottom=y+20;
  x=TextRect.left=TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN;
  TextRect.right=x+RSAWindowCols;
  DrawBrush=CreateSolidBrush(RGB(255,255,255));
  FillRect(hDC,&TextRect,DrawBrush);
  DeleteObject(DrawBrush);
  if ((((int)Time)%60000)/1000 < 10)
    sprintf(text,"Elapsed time (min:sec): %3d:0%1d ",(int)Time/60000,(((int)Time)%60000)/1000);
  else
    sprintf(text,"Elapsed time (min:sec): %3d:%2d ",(int)Time/60000,(((int)Time)%60000)/1000);
  GetTextExtentPoint32(hDC,text,strlen(text),&TextSize);
  TextOut(hDC,x+RSAWindowCols/2-TextSize.cx/2,y,text,strlen(text));
  x=10;
  y=TOP_WINDOW_MARGIN;
  sprintf(text,"RSA mean: %5.1lf   ",MeanRSA);
  TextOut(hDC,x,y+110,text,strlen(text));
  sprintf(text,"RSA std dev: %5.1lf   ",StdDevRSA);
  TextOut(hDC,x,y+130,text,strlen(text));
  sprintf(text,"RSA current: %5.1lf   ",CurrentRSA);
  TextOut(hDC,x,y+150,text,strlen(text));
  sprintf(text,"Total minutes: %d   ",TotalMinutes);
  TextOut(hDC,x,y+170,text,strlen(text));
  ReleaseDC(hWnd,hDC);
  }

if (DisplayMode == 3)
  {		  /* balloon */
  hDC=GetDC(hWnd);
  GetClientRect(hWnd,&WindowRect);
  x=WindowRect.right/2; y=WindowRect.bottom/2;
			  /* clip ZScore to interested range (from display settings) */
  GaugeReading=Arousal;
  if (GaugeReading < (0.0-ZScoreRange))
	GaugeReading=0.0-ZScoreRange;
  if (GaugeReading > ZScoreRange)
	GaugeReading=ZScoreRange;
			  /* normalize reading to 0...1.0 as percentage of total interested range */
  NormZ=((GaugeReading-(0.0-ZScoreRange))/(ZScoreRange-(0.0-ZScoreRange)));
			  /* calculate balloon radius based on normalized reading */
  radius=(int)(NormZ*(double)(WindowRect.right-WindowRect.left)/2.0);
  if (radius < last_radius)
	{		  /* erase previous circle(s) */
	MoveToEx(hDC,x+last_radius,y,NULL);
	DrawPen=CreatePen(PS_SOLID,(last_radius-radius)*2,RGB(255,255,255));
	OrigPen=SelectObject(hDC,DrawPen);
	AngleArc(hDC,x,y,last_radius,0,360);
	SelectObject(hDC,OrigPen);
	DeleteObject(DrawPen);
	}
  DrawBrush=CreateSolidBrush(RGB(0,0,255));	  // pink??
  OrigBrush=SelectObject(hDC,DrawBrush);
  Ellipse(hDC,x-radius,y-radius,x+radius,y+radius);
  SelectObject(hDC,OrigBrush);
  DeleteObject(DrawBrush);
  ReleaseDC(hWnd,hDC);
  last_radius=radius;
  }

if (DisplayMode == 4)
  {
  DisplayHeartrate(hWnd,100,100,(int)HeartWindowX,0,sIBI,
					LocalSamplingRate,300,1000);
  DisplayRSA(hWnd,100,500,(int)HeartWindowX,1,CurrentRSA,
					LocalSamplingRate,300,1000);
		/* update X-axis display location */
  HeartWindowX+=(((double)1000.0/(double)PlotDataRange)/(double)LocalSamplingRate);
  if ((int)HeartWindowX > 1000)
	HeartWindowX=0.0;
  }

if (DisplayMode == 2)
  {
  DisplayGauge(hWnd,600,500,Arousal,400);
  }

}




void DisplayHeartrate(HWND hWnd,		/* the window to display in */
					  int LeftX,		/* the left of the heartrate window */
					  int TopY,			/* the top of the heartrate window */
					  int HeartX,		/* the specific x-axis location to draw */
					  int InvertData,	/* 0=>display heartrate (units are BPM), 1=>display IBI (units are MPB) */
					  int sIBI,			/* the data point (units are MPB) */
					  int LocalSamplingRate,  /* used to scale the X axis appropriately */
					  int WindowRows,	/* total height of heartrate window */	
					  int WindowCols)	/* total width of heartrate window */

{
int		x,y;
HDC		hDC;
HBRUSH	DrawBrush;
RECT	TextRect;

		/* erase old data displayed at this vertical space */
hDC=GetDC(hWnd);
y=TextRect.top=TopY;			  /* TopY = TOP_WINDOW_MARGIN */
TextRect.bottom=TopY+WindowRows;  /* LeftX = TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN */
x=TextRect.left=LeftX+HeartX;
TextRect.right=x+30;		/* forward erase 30 pixels - leaves nice margin */
if (TextRect.right > LeftX+WindowCols)	/* clip erase area if off right boundary */
  TextRect.right=LeftX+WindowCols;
DrawBrush=CreateSolidBrush(RGB(200,200,200)); /* erase (grey) */
FillRect(hDC,&TextRect,DrawBrush);
DeleteObject(DrawBrush);
		/* figure out Y-position to put new data point */
if (InvertData)
  y=TextRect.top=TopY+WindowRows-
				  (int)((double)(sIBI-PlotDataMinValue)/(double)(PlotDataMaxValue-PlotDataMinValue)*(double)WindowRows)-
				  TICKMARK_THICKNESS/2;
else
  y=TextRect.top=TopY+
				  (int)((double)(sIBI-PlotDataMinValue)/(double)(PlotDataMaxValue-PlotDataMinValue)*(double)WindowRows)-
				  TICKMARK_THICKNESS/2;

TextRect.bottom=y+TICKMARK_THICKNESS;
x=TextRect.left=LeftX+(int)HeartX;
TextRect.right=x+((WindowCols/PlotDataRange)/LocalSamplingRate);
		  /* check for clipping on window boundaries */
if (TextRect.right == TextRect.left) TextRect.right=TextRect.left+3;
if (TextRect.right > LeftX+WindowCols)
  TextRect.right=LeftX+WindowCols;
if (TextRect.top < TopY)
  { TextRect.top=TopY; TextRect.bottom=TopY+TICKMARK_THICKNESS/2; }
if (TextRect.bottom > TopY+WindowRows)
  { TextRect.bottom=TopY+WindowRows; TextRect.top=TopY+WindowRows-TICKMARK_THICKNESS/2; }
DrawBrush=CreateSolidBrush(RGB(255,0,0));
FillRect(hDC,&TextRect,DrawBrush);
DeleteObject(DrawBrush);
ReleaseDC(hWnd,hDC);
}




void DisplayRSA(HWND hWnd,				/* the window to display in */
				int LeftX,				/* the left of the RSA window */
				int TopY,				/* the top of the RSA window */
				int AxisX,				/* the specific x-axis location to draw */
				int InvertData,			/* 0=>larger power = higher, 1=>larger power = lower */
				double RSA,				/* the data point */
				int LocalSamplingRate,  /* used to scale the X axis appropriately */
				int WindowRows,			/* total height of RSA window */	
				int WindowCols)			/* total width of RSA window */

{
int		x,y;
HDC		hDC;
HBRUSH	DrawBrush;
RECT	TextRect;

		/* erase window around where we are about to plot RSA */
hDC=GetDC(hWnd);
y=TextRect.top=TopY;			  /* TopY = TOP_WINDOW_MARGIN+HeartRateWindowRows+TOP_WINDOW_MARGIN */
TextRect.bottom=TopY+WindowRows;  /* LeftX = TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN */
x=TextRect.left=LeftX+AxisX;
TextRect.right=x+30;		/* forward erase 30 pixels - leaves nice margin */
if (TextRect.right > LeftX+WindowCols)	/* clip erase area if off right boundary */
  TextRect.right=LeftX+WindowCols;
DrawBrush=CreateSolidBrush(RGB(200,200,200)); /* erase (grey) */
FillRect(hDC,&TextRect,DrawBrush);
DeleteObject(DrawBrush);
		/* figure out Y-position to put new data point */
if (InvertData)
  y=TextRect.top=TopY+
				  (int)((RSA-(double)PowerDataMinValue)/
				  (double)(PowerDataMaxValue-PowerDataMinValue)*(double)WindowRows)-
				  TICKMARK_THICKNESS/2;
else
  y=TextRect.top=TopY+WindowRows-
				  (int)((RSA-(double)PowerDataMinValue)/
				  (double)(PowerDataMaxValue-PowerDataMinValue)*(double)WindowRows)-
				  TICKMARK_THICKNESS/2;

TextRect.bottom=y+TICKMARK_THICKNESS;
x=TextRect.left=LeftX+(int)AxisX;
TextRect.right=x+((WindowCols/RSADataRange)/LocalSamplingRate);
		  /* check for clipping on window boundaries */
if (TextRect.right == TextRect.left) TextRect.right=TextRect.left+1;
if (TextRect.right > LeftX+WindowCols)
  TextRect.right=LeftX+WindowCols;
if (TextRect.top < TopY)
  { TextRect.top=TopY; TextRect.bottom=TopY+TICKMARK_THICKNESS/2; }
if (TextRect.bottom > TopY+WindowRows)
  { TextRect.bottom=TopY+WindowRows; TextRect.top=TopY+WindowRows-TICKMARK_THICKNESS/2; }
DrawBrush=CreateSolidBrush(RGB(100,200,100));
FillRect(hDC,&TextRect,DrawBrush);
DeleteObject(DrawBrush);
ReleaseDC(hWnd,hDC);
}






void DisplayGauge(HWND hWnd,				/* the window to display in */
				  int CenterX,				/* the X-position of the horizontal center of the gauge */
				  int BottomY,				/* the Y-position of the bottom of the gauge (center line of circle) */
				  double Arousal,			/* the data point */
				  int GaugeRadius)			/* radius of the gauge */

{
int		x,y;
HDC		hDC;
HBRUSH	DrawBrush;
RECT	TextRect,ArrowRect;
double	GaugeReading,NormZ;
int		i;

			  /* display gauge reading */
			  /* clip ZScore to displayable range and normalize to scale to degrees */
GaugeReading=Arousal;
if (GaugeReading < (0.0-ZScoreRange))
  GaugeReading=0.0-ZScoreRange;
if (GaugeReading > ZScoreRange)
  GaugeReading=ZScoreRange;
NormZ=/*1.0-*/((GaugeReading-(0.0-ZScoreRange))/(ZScoreRange-(0.0-ZScoreRange)));
				/* code to actually display the ameter reading */
hDC=GetDC(hWnd);
y=TextRect.top=BottomY-GaugeRadius;		  /* BottomY = TOP_WINDOW_MARGIN+HeartRateWindowRows */
TextRect.bottom=y+GaugeRadius*2;
x=TextRect.left=CenterX-GaugeRadius;	  /* CenterX = TEXT_DISPLAY_WIDTH+SIDE_WINDOW_MARGIN+HeartRateWindowCols+SIDE_WINDOW_MARGIN+RSAWindowCols/2 */
TextRect.right=x+GaugeRadius*2;
Pie(hDC,x,y,x+GaugeRadius*2,y+GaugeRadius*2, x+GaugeRadius*2,y+GaugeRadius,x,y+GaugeRadius);
y=TextRect.top=BottomY;
x=CenterX;
DrawBrush=CreateSolidBrush(RGB(100,200,100));
for (i=0; i<GaugeRadius-2; i++)
  {
  ArrowRect.bottom=y-(int)((double)i*sin((1.0-NormZ)*M_PI));
  ArrowRect.left=x+(int)((double)i*cos((1.0-NormZ)*M_PI));
  ArrowRect.top=ArrowRect.bottom-2;
  ArrowRect.right=ArrowRect.left+2;
  FillRect(hDC,&ArrowRect,DrawBrush);
  }
DeleteObject(DrawBrush);
ReleaseDC(hWnd,hDC);
}

