
/* This work is described in "A Real-Time Index of Vagal Activity", by
** A. Hoover and E. Muth, in International Journal of Human-Computer
** Interaction, accepted November 2003, to appear. */

/* based upon a method described in "Significance of Autonomic Nervous
** System Activity in Functional Dyspepsia", by E. Muth et. al., in
** Digestive Diseases and Sciences, vol 45 no 5, May 2000, pp 854-863. */

/* further details are planned for future publication */

	/* Coded by Adam Hoover, May 2002 */
	/* feedback/modified June-July 2002 */
	/* note:  writeup in paper does not match code from alligator-tool */
	/* ported to laptop demo August 2002 */
	/* packaged under Gnu General Public License Nov 2002 */
	/* added serial port interface to EZ-IBI module Jan 2003 */
	/* added IBI error detection via clipping Aug 2003 */
	/* various kludges for supporting various research groups, Jan-Aug 2003 */
	/* repackaged for library support Nov-Dec 2003 */
	/* IBI buffer error detection/correction Nov-Jan 2003 */
	/* log-normal distribution normalization Jan 2003 */

			/* 2.0 changes */
	/* added IBI error detection & correction functions, version 1 */
	/* rewrote software for better modularity, for WAM and IHMC */
	/* added log-normal distribution normalization */

			/* 2.1 changes */
	/* eliminated power history smoothing; uses less memory (for WAM) */
			/* 2.1b changes */
	/* WAM only - replaced all constant variables with constants */

			/* 2.2 changes */
	/* changed CPM integerizing method that (stupidly) doubled 2 frequencies */
			/* 2.2b changes */
	/* fixed bug in playback of non-AMeter data (plain IBIs) */
	/* fixed bug in version# output to files */
			/* 2.2c changes */
	/* added playback with no graphics option, for super-fast playback */

			/* 2.3 changes */
	/* added Kalman filter smoothing to arousal output */
	/* fixed bugs for inputting calibration values (changed int code to double code) */
	/* fixed bug that required WindowSize+1 before going live; now takes WindowSize */
	/* completely eliminated power averaging stuff from code */

			/* 2.4 changes (May 2005) */
	/* added in Jeromie Rand's IBI error detection & correction engine (see his thesis for details) */
	/*	included limit on sequential corrections (=3) */
	/* fixed bug on playback speed (stays max-no-graphics if selected) */
	/* added option to run with or without continuous recalibration */

			/* 2.4b changes (October 2005) */
	/* added GUI option for turning ED&C engine on or off during processing */

			/* 2.5 changes (December 2005 - January 2006) */
	/* added various graphic displays for arousal meter feedback (gauge, balloon, streaming) */
	/* fixed bug in speeding up "playback" of test signal */
	/* test signal no longer writes any files */

/* Copyright (C) 2002-2006, Adam Hoover and Eric Muth */

/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The authors of this software can be contacted at the Electrical and Computer
Engineering Dept., Clemson University, Clemson, SC 29634-0915 USA.
*/


#include <stdio.h>		/* standard I/O library (e.g. printf function) */
#include <windows.h>	/* standard Windows library (e.g. mutex and MessageBox functions) */
#include <process.h>	/* thread and process functions (e.g. _beginthread function) */
#include <math.h>		/* standard math library (e.g. log function) */
#include <time.h>		/* for timing functions (e.g. clock function) */
#include "globals.h"	/* program global variables */
#include "resource.h"	/* generated by compiler - macros and prototypes for GUI stuff */

void Monitor(HWND hWnd)

{
		/* timing variables for synchronous operations */
clock_t				  MainStart;			  /* time this process starts */
												  /* used to calculate a cumulative elapsed, */
												  /* more precise than an iteration's elapsed */
clock_t				  Start;				  /* time each iteration starts */
clock_t				  Finish;				  /* time each iteration's computations finish */
int					  Elapsed;				  /* sleep period for syncing each iteration */
int					  ApproximateElapsed;	  /* sleep for playback iteration (not precise) */
clock_t				  WaitStart;			  /* time that the sleep for each iteration starts */
int					  SampleTimeInMS;		  /* milliseconds of a synchronous sampling period */
int					  WarningOnSamplingRate;  /* time to display warning if sync timing lags */
												  /* (e.g. this process gets blocked too long) */
int					  Time;		/* perfect master "clock" - synced to local processor time */
								/* really just a counter - only increments by 250ms "ticks" */
								/* units are ms (for convenience), starts at zero */

		/* arousal analysis variables */
int					  aIBI;					  /* current asynchronous (raw) uncorrected IBI */
int					  aIBI_timestamp;		  /* local timestamp of current aIBI */
int					  sIBI;					  /* current synchronous, corrected IBI */
int					  TotalData;			  /* total sIBI's buffered for analysis */
double				  Power[TOTAL_FREQS];	  /* current power spectrum result from analysis */
double				  MaxPower;				  /* power of max band */
int					  MaxPowerCPM;			  /* CPM of current max power band */
double				  Arousal;				  /* current arousal */

		/* variables for reading/writing IBIs and results from/to files */
FILE				  *fpt,*fptw_raw,*fptw_aro;
char				  TimeText[100],DateText[100];
char				  text[300];
int					  j;

		/* variables for communicating other processes */
double				  *SharedMemoryPointer;	  /* place to send arousal */




		/* start up IBI detector thread */
new_aIBI_flag=0;	/* this flag should set when a new aIBI is detected */
if (AnalysisMode == 0)
  {		/* analyzing live input; put your IBI detecting function here */
  _beginthread(EZIBI,0,hWnd);
  }
if (AnalysisMode == 2)
  {		/* replaying saved asynchronous IBI data from file */
  SetUpPlayback();
  }

		/* map shared memory (system page) to pointer */
SharedMemoryPointer=MapViewOfFile(SharedMemoryHandle,FILE_MAP_WRITE,0,0,0);
*SharedMemoryPointer=0.0; /* intialize to 0 for output */

		/* set timezone parameter from O/S query (this is used later to output system time) */
_tzset();

		/* open files for writing of data (for non-playback modes) */
if (AnalysisMode == 0)
  {		/* find next data file name that isn't used yet, and write out data */
  j=0;
  while (j < 10000)
	{	/* make sure both files (dataN-raw.txt and dataN-aro.txt) do not exist already */
    sprintf(text,"C:\\Data\\data%d-raw.txt",j);
    if ((fpt=fopen(text,"r")) != NULL)
	  {
	  fclose(fpt);
	  j++;
	  continue;
	  }
    sprintf(text,"C:\\Data\\data%d-aro.txt",j);
    if ((fpt=fopen(text,"r")) != NULL)
	  {
	  fclose(fpt);
	  j++;
	  continue;
	  }
    break;
	}
  sprintf(text,"C:\\Data\\data%d-raw.txt",j);
  if ((fptw_raw=fopen(text,"wb")) == NULL)
	{
	MessageBox(NULL,"Unable to open for saving raw IBI data",text,MB_OK | MB_APPLMODAL);
	PostMessage(MainWnd,WM_COMMAND,ID_STOP,0);
	MonitorOn=0;
	}
  sprintf(text,"C:\\Data\\data%d-aro.txt",j);
  if ((fptw_aro=fopen(text,"wb")) == NULL)
	{
	MessageBox(NULL,"Unable to open for saving arousal data",text,MB_OK | MB_APPLMODAL);
	PostMessage(MainWnd,WM_COMMAND,ID_STOP,0);
	MonitorOn=0;
	}
  _strtime(TimeText);
  _strdate(DateText);
  fprintf(fptw_raw,"# AMeter raw aIBI data Version %s\n",VersionText);
  fprintf(fptw_raw,"# %s %s\n",DateText,TimeText);
  fprintf(fptw_raw,"# SyncTime(ms)  TimeStamp(ms)  aIBI\n");
  fprintf(fptw_aro,"# AMeter results Version %s\n",VersionText);
  fprintf(fptw_aro,"# %s %s\n",DateText,TimeText);
  fprintf(fptw_aro,"# SyncTime(ms)  sIBI  Arousal  MaxPower\n");
  }
		/* open files for writing of data (playback modes) */
if (AnalysisMode == 2) /* reading data already saved, so don't resave aIBIs */
  {		/* don't mind overwriting - just open */
  strcpy(text,PlaybackFilename);
  j=strlen(text)-1;
  while (j > 5  &&  text[j] != '-'  &&  text[j] != '\\'  &&  text[j] != '/')
	j--;
  if (j == 5  ||  text[j] == '\\'  ||  text[j] == '/')
	{
    j=strlen(text)-1;
    while (j > 5  &&  text[j] != '.'  &&  text[j] != '\\'  &&  text[j] != '/')
	  j--;
	if (j == 5  ||  text[j] == '\\'  ||  text[j] == '/')
	  j=strlen(text)-1;
	}
  text[j]='\0';
  strcat(text,"-replay-aro.txt");
  if ((fptw_aro=fopen(text,"wb")) == NULL)
	{
	MessageBox(NULL,"Unable to open for saving replay of arousal data",text,MB_OK | MB_APPLMODAL);
	PostMessage(MainWnd,WM_COMMAND,ID_STOP,0);
	MonitorOn=0;
	}
  _strtime(TimeText);
  _strdate(DateText);
  fprintf(fptw_aro,"# AMeter replay results Version %s\n",VersionText);
  fprintf(fptw_aro,"# %s %s\n",DateText,TimeText);
  fprintf(fptw_aro,"# Time(ms) sIBI Arousal MaxPower\n");
  }

		/* total sIBI's buffered for analysis */
TotalData=0;

		/* clock for this program (in ms) */
Time=0;
WarningOnSamplingRate=0;	/* ms that warning will be displayed when sync timing goes off */
MaxPower=Arousal=sIBI=0;	/* to avoid compiler warning - not necessary here */

		/* initialize buffer; the -1 in the aIBI position does this */
AddToBuffer(-1,0);

		/* initialize local analysis variables; sIBI=-1 does this, everything else ignored */
ArousalAnalysis(-1,SamplingRate,WindowSize,ArousalSmooth,&TotalData,
				  Power,&MaxPower,&MaxPowerCPM,&MeanRSA,&StdDevRSA,&TotalMinutes,&Arousal);

		/* initialize display; the -1 in the sIBI position does this; everything else ignored */
DisplayResult(hWnd,DisplayMode,-1,SamplingRate,Time,WindowSize-TotalData,Power,
				WindowSize-TotalData,Arousal,MeanRSA,StdDevRSA,MaxPower,
				TotalMinutes);

		/* initialize MainStart, used to sync total time (synchronous time) while running */
MainStart=clock();

while (MonitorOn)
  {
		/* iteration start time stamp; only used for feedback, for imprecise speedup */
  Start=clock();

		/* check for new aIBI */
  if (AnalysisMode == 0  &&  new_aIBI_flag == 1)
	{
	aIBI=new_aIBI_value;
	aIBI_timestamp=new_aIBI_timestamp;
	}
  else if (AnalysisMode == 1  &&  IBI_TestSignal(Time,1000/SamplingRate))
	{
	aIBI=new_aIBI_value;
	aIBI_timestamp=new_aIBI_timestamp;
	}
  else if (AnalysisMode == 2  &&  CheckPlayback(Time))
	{
	aIBI=new_aIBI_value;
	aIBI_timestamp=new_aIBI_timestamp;
	}

		/* process new aIBI */
  if (new_aIBI_flag == 1)
	{
		/* display raw (uncorrected) aIBI in GUI */
	if (DisplayMode == 0  ||  DisplayMode == 1)
	  Display_aIBI(hWnd,aIBI,WarningOnSamplingRate);
		/* output raw (uncorrected) aIBI to file if not running a playback */
    if (AnalysisMode == 0)
	  {
	  if (fprintf(fptw_raw,"%8d  %8d  %5d\n",Time,aIBI_timestamp,aIBI) < 0)
		{
	    MessageBox(NULL,"Error writing raw IBI","File too large?  Disk full?",MB_OK | MB_APPLMODAL);
	    PostMessage(MainWnd,WM_COMMAND,ID_STOP,0);
	    MonitorOn=0;
	    continue;
		}
	  }
		/* update buffer with new aIBI, which will also error detect & correct */
	AddToBuffer(aIBI,aIBI_timestamp);
	new_aIBI_flag=0;
	}

  if (Time > 7999)		/* wait for aIBI buffer to get 8 seconds of data */
	{
		/* get synchronous IBI */
    j=GetSyncIBI(&sIBI,1000/SamplingRate);

		/* analyze arousal */
    j=ArousalAnalysis(sIBI,SamplingRate,WindowSize,ArousalSmooth,&TotalData,
				  Power,&MaxPower,&MaxPowerCPM,&MeanRSA,&StdDevRSA,&TotalMinutes,&Arousal);

		/* display results */
    DisplayResult(hWnd,DisplayMode,sIBI,SamplingRate,Time,WindowSize-TotalData,Power,
				j,Arousal,MeanRSA,StdDevRSA,MaxPower,TotalMinutes);

		/* output 1Hz results [timestamp corrected-sIBI arousal RSA-power] to file */
	if (AnalysisMode == 0  ||  AnalysisMode == 2)
      fprintf(fptw_aro,"%d %d %5.2lf %.1lf\n",Time,sIBI,Arousal,MaxPower);

		/* send AMeter gauge reading over shared memory space */
    *SharedMemoryPointer=Arousal;
	}

		/* snyc timing */
			  /* compute time to wait based upon SamplingRate, and wait it */
			  /* Time = synchrnous (ideal) clock at SamplingRate */
			  /* Elapsed = time measured for this one iteration based on larger calculation of total running time */
			  /* ApproximateElapsed = simple time measured for this one iteration, but has roundoff error - only used for playback */
  Time=Time+(1000/SamplingRate);
  SampleTimeInMS=1000/SamplingRate;
  Finish=clock();
  ApproximateElapsed=(int)((double)(Finish-Start)/(double)CLOCKS_PER_SEC*1000);	/* time for single iteration (not precise) */
  Elapsed=(int)((double)(Finish-MainStart)/(double)CLOCKS_PER_SEC*1000.0);	/* time since whole thing started */
  Elapsed-=(Time-(1000/SamplingRate)); /* subtract out ideal time for all iterations minus this one */
			  /* check if sampling rate maintained on real signal, and fake it otherwise, warning user */
  if (WarningOnSamplingRate > 0)
	WarningOnSamplingRate-=SampleTimeInMS;
  if (AnalysisMode < 2  &&  Elapsed > SampleTimeInMS)
	{		  /* timing wasn't maintained - update buffer with copy(s) of most recent IBI */
	while (Elapsed > SampleTimeInMS)
	  {
		// used to have to update buffer here; now async IBI detector should handle update?
	  Elapsed-=SampleTimeInMS;
	  Time+=SampleTimeInMS;
	  WarningOnSamplingRate+=500; /* display warning for 1/2 second for each missed interval */
	  }
	}
  WaitStart=clock();
  if (AnalysisMode == 0)	/* live analysis mode, precise waiting */
    while (Elapsed+(((int)clock()-(int)WaitStart)*CLOCKS_PER_SEC/1000) < SampleTimeInMS) ;
  else if (AnalysisMode >= 1)	  /* playback mode, allow speedup (clock is not precise) */
    while (ApproximateElapsed+(((int)clock()-(int)WaitStart)*CLOCKS_PER_SEC/1000) < SampleTimeInMS/PlaybackSpeed) ;

  }			  /* end while (MonitorOn) main loop */


//if (AnalysisMode == 2)
//  MessageBox(NULL,"Playback complete","Done",MB_OK | MB_APPLMODAL);

		/* close down analysis local variables; sIBI=-2 does this, everything else ignored */
ArousalAnalysis(-2,SamplingRate,WindowSize,ArousalSmooth,&TotalData,
				  Power,&MaxPower,&MaxPowerCPM,&MeanRSA,&StdDevRSA,&TotalMinutes,&Arousal);

		/* close output files */
if (AnalysisMode == 0  ||  AnalysisMode == 2)
  fclose(fptw_aro);
if (AnalysisMode == 0)
  fclose(fptw_raw);

		/* unmap shared memory */
UnmapViewOfFile(SharedMemoryPointer);

MonitorOn=0;
PostMessage(MainWnd,WM_COMMAND,ID_STOP,0);	/* fixes GUI */

_endthread();
return;
}

